"use strict";
const ts = require("typescript");
const path = require("path");
const babel = require("babel-core");
const fs_extra_p_1 = require("fs-extra-p");
const bluebird_1 = require("bluebird");
const declarationGenerator_1 = require("./declarationGenerator");
const docGenerator_1 = require("./docGenerator");
const __awaiter = require("./awaiter");
const basePath = process.argv.length === 3 ? path.resolve(process.argv[2]) : process.cwd();
const tsConfigPath = path.join(basePath, "tsconfig.json");
main()
    .catch(error => {
    if (error instanceof CompilationError) {
        for (let diagnostic of error.errors) {
            if (diagnostic.file == null) {
                console.log(diagnostic.messageText);
                continue;
            }
            const location = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
            const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
            console.log(`${diagnostic.file.fileName} (${location.line + 1},${location.character + 1}): ${message}`);
        }
    }
    else {
        console.error(error.stack || error.message || error);
    }
    process.exit(-1);
});
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const jsonResult = ts.parseConfigFileTextToJson(tsConfigPath, yield fs_extra_p_1.readFile(tsConfigPath, "utf8"));
        if (jsonResult.error != null) {
            throw new CompilationError([jsonResult.error]);
        }
        const result = ts.parseJsonConfigFileContent(jsonResult.config, {
            readDirectory: ts.sys.readDirectory
        }, basePath);
        checkErrors(result.errors);
        yield compile(result, jsonResult.config);
    });
}
function compile(config, tsConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        const compilerOptions = config.options;
        const declarationConfig = tsConfig.declaration;
        if (declarationConfig != null) {
            compilerOptions.declaration = true;
        }
        compilerOptions.noEmitOnError = true;
        const program = ts.createProgram(config.fileNames, compilerOptions, ts.createCompilerHost(compilerOptions));
        checkErrors(ts.getPreEmitDiagnostics(program));
        const outDir = compilerOptions.outDir;
        if (outDir == null) {
            throw new Error("outDir is not specified in the compilerOptions");
        }
        yield fs_extra_p_1.ensureDir(outDir);
        const target = compilerOptions.target || ts.ScriptTarget.Latest;
        const fileToSourceMap = {};
        const promises = [];
        const emittedFiles = new Set();
        const declarationFiles = Array();
        const emitResult = program.emit(undefined, (fileName, data) => {
            emittedFiles.add(fileName);
            if (fileName.endsWith(".js")) {
                const sourceMapFileName = fileName + ".map";
                processCompiled(data, fileToSourceMap[sourceMapFileName], fileName, sourceMapFileName, promises);
            }
            else if (fileName.endsWith(".js.map")) {
                fileToSourceMap[fileName] = data;
            }
            else if (declarationConfig != null) {
                declarationFiles.push(ts.createSourceFile(fileName, data, target, true));
            }
        });
        if (tsConfig.docs != null) {
            const docs = docGenerator_1.generateDocs(program);
            if (docs.length !== 0) {
                docGenerator_1.writeDocFile(path.resolve(basePath, tsConfig.docs), docs);
            }
        }
        checkErrors(emitResult.diagnostics);
        if (emitResult.emitSkipped) {
            throw new Error("Emit skipped");
        }
        if (declarationFiles.length > 0) {
            for (let moduleName of Object.keys(declarationConfig)) {
                promises.push(declarationGenerator_1.generateDeclarationFile(moduleName, declarationFiles, compilerOptions, path.join(basePath, declarationConfig[moduleName]), basePath));
            }
        }
        yield bluebird_1.Promise.all(promises);
        promises.length = 0;
        yield removedOld(outDir, emittedFiles, promises);
        yield bluebird_1.Promise.all(promises);
    });
}
function removedOld(outDir, emittedFiles, promises) {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield fs_extra_p_1.readdir(outDir);
        for (let file of files) {
            if (file[0] !== "." && !file.endsWith(".d.ts")) {
                const fullPath = outDir + '/' + file;
                if (!file.includes(".")) {
                    removedOld(fullPath, emittedFiles, promises);
                    continue;
                }
                if (!emittedFiles.has(fullPath)) {
                    promises.push(fs_extra_p_1.unlink(fullPath));
                }
            }
        }
    });
}
function processCompiled(code, sourceMap, jsFileName, sourceMapFileName, promises) {
    const result = babel.transform(code, {
        inputSourceMap: JSON.parse(sourceMap),
        sourceMaps: true,
        filename: jsFileName,
    });
    promises.push(fs_extra_p_1.outputFile(jsFileName, result.code), fs_extra_p_1.outputJson(sourceMapFileName, result.map));
}
function checkErrors(errors) {
    if (errors.length !== 0) {
        throw new CompilationError(errors);
    }
}
class CompilationError extends Error {
    constructor(errors) {
        super("Compilation error");
        this.errors = errors;
    }
}
//# sourceMappingURL=builder.js.map