"use strict";
const ts = require("typescript");
const fs_extra_p_1 = require("fs-extra-p");
const markdown = require("markdown-it");
const __awaiter = require("./awaiter");
function generateDocs(program) {
    const topicToProperties = new Map();
    for (let sourceFile of program.getSourceFiles()) {
        if (!sourceFile.isDeclarationFile) {
            for (let statement of sourceFile.statements) {
                if (statement.kind === ts.SyntaxKind.InterfaceDeclaration) {
                    const interfaceDescription = getComment(statement);
                    if (interfaceDescription == null) {
                        continue;
                    }
                    const interfaceDeclaration = statement;
                    const interfaceName = interfaceDeclaration.name.text;
                    const interfaceDescriptor = {
                        name: interfaceName,
                        description: interfaceDescription,
                    };
                    let nameToProperty = topicToProperties.get(interfaceDescriptor);
                    for (let member of interfaceDeclaration.members) {
                        if (member.kind === ts.SyntaxKind.PropertySignature) {
                            const comment = getComment(member);
                            if (comment != null) {
                                if (nameToProperty == null) {
                                    nameToProperty = new Map();
                                    topicToProperties.set(interfaceDescriptor, nameToProperty);
                                }
                                const symbol = member.symbol;
                                nameToProperty.set(member.name.text, new PropertyDescriptor(interfaceName, comment, symbol != null && (symbol.flags & ts.SymbolFlags.Optional) !== 0));
                            }
                        }
                    }
                }
            }
        }
        function getComment(node) {
            const leadingCommentRanges = ts.getLeadingCommentRanges(sourceFile.text, node.pos);
            if (leadingCommentRanges == null || leadingCommentRanges.length === 0) {
                return null;
            }
            else {
                const commentRange = leadingCommentRanges[0];
                if (sourceFile.text[commentRange.pos] == "/" && sourceFile.text[commentRange.pos + 1] == "/") {
                    return null;
                }
                else {
                    const isTwo = sourceFile.text[commentRange.pos + 2] == "*";
                    return stripIndent(sourceFile.text.slice(commentRange.pos + (isTwo ? 3 : 2), commentRange.end - "*/".length)).trim();
                }
            }
        }
    }
    return renderDocs(topicToProperties);
}
exports.generateDocs = generateDocs;
function writeDocFile(docOutFile, content) {
    return __awaiter(this, void 0, void 0, function* () {
        let existingContent = null;
        try {
            existingContent = yield fs_extra_p_1.readFile(docOutFile, "utf8");
        }
        catch (e) {
        }
        if (existingContent == null) {
            return fs_extra_p_1.outputFile(docOutFile, content);
        }
        else {
            const startMarker = "<!-- do not edit. start of generated block -->";
            const endMarker = "<!-- end of generated block -->";
            const start = existingContent.indexOf(startMarker);
            const end = existingContent.indexOf(endMarker);
            if (start != -1 && end != -1) {
                return fs_extra_p_1.outputFile(docOutFile, existingContent.substring(0, start + startMarker.length) + "\n" + content + "\n" + existingContent.substring(end));
            }
        }
        console.log("Write doc to " + docOutFile);
    });
}
exports.writeDocFile = writeDocFile;
function stripIndent(str) {
    const match = str.match(/^[ \t]*(?=\S)/gm);
    const indent = match == null ? 0 : Math.min.apply(Math, match.map(it => it.length));
    return indent > 0 ? str.replace(new RegExp("^[ \\t]{" + indent + "}", "gm"), "") : str;
}
function renderDocs(topicToProperties) {
    let result = "";
    const md = markdown({
        typographer: true,
    })
        .disable(["link", "emphasis"]);
    function render(src) {
        return src.includes("\n") ? md.render(src).trim().replace(/\n/g, " ") : src;
    }
    topicToProperties.forEach((nameToProperty, interfaceDescriptor) => {
        result += `\n${anchor(interfaceDescriptor.name)}\n${interfaceDescriptor.description}\n`;
        if (interfaceDescriptor.description.includes("\n")) {
            result += "\n";
        }
        result += "| Name | Description\n";
        result += "| --- | ---";
        nameToProperty.forEach((descriptor, propertyName) => {
            var bold = descriptor.isOptional ? "" : "**";
            result += `\n| ${bold}${propertyName}${bold} | `;
            result += anchor(descriptor.interfaceName + "-" + propertyName);
            result += render(descriptor.description);
        });
        result += "\n";
    });
    return result;
}
function anchor(link) {
    return `<a name="${link}"></a>`;
}
class InterfaceDescriptor {
}
exports.InterfaceDescriptor = InterfaceDescriptor;
class PropertyDescriptor {
    constructor(interfaceName, description, isOptional) {
        this.interfaceName = interfaceName;
        this.description = description;
        this.isOptional = isOptional;
    }
}
exports.PropertyDescriptor = PropertyDescriptor;
//# sourceMappingURL=docGenerator.js.map